import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import javax.swing.*;
import java.net.*;

// send message switch to client

/**
 * A ChatRoomClientGUI  is a JPanel that displaya the UI of a chat room client
 */
public class ChatRoomClientGUI extends JPanel {

    public static ResourceManager resources = ResourceManager.getInstance();

	// the image files
    public static final ImageIcon CONNECT_IMAGE     = resources.getImageIcon("CONNECT_IMAGE");
    public static final ImageIcon DISCONNECT_IMAGE  = resources.getImageIcon("DISCONNECT_IMAGE");
    public static final ImageIcon CLEAR_IMAGE       = resources.getImageIcon("CLEAR_IMAGE");
    public static final ImageIcon COLOR_IMAGE       = resources.getImageIcon("COLOR_IMAGE");
    public static final ImageIcon PLUS_IMAGE        = resources.getImageIcon("PLUS_IMAGE");
    public static final ImageIcon MINUS_IMAGE       = resources.getImageIcon("MINUS_IMAGE");
    public static final ImageIcon ABOUT_IMAGE       = resources.getImageIcon("ABOUT_IMAGE");
    public static final ImageIcon EXIT_IMAGE        = resources.getImageIcon("EXIT_IMAGE");
    public static final ImageIcon CLIENT_ICON       = resources.getImageIcon("CLIENT_ICON");

    // sounds
    public static final AudioClip CHAT_SOUND        = resources.getAudioClip("CHAT_SOUND");

    // string messages
    public static final String    INPUT_MESSAGE     = resources.getString("INPUT_MESSAGE");
    public static final String    CHAT_ROOM         = resources.getString("CHAT_ROOM");
    public static final String    CHAT_ROOM_CLIENT  = resources.getString("CHAT_ROOM_CLIENT");
    public static final String    CON_INFO_STR      = resources.getString("CON_INFO_STR");
    public static final String    ENTER_NAME_STR    = resources.getString("ENTER_NAME_STR");
    public static final String    SERVER_REQUEST_STR= resources.getString("SERVER_REQUEST_STR");
    public static final String    EXIT_CLIENT_STR    = resources.getString("EXIT_CLIENT_STR");
    public static final String    DISPLAY_INFO_STR   = resources.getString("DISPLAY_INFO_STR");
    public static final String    APP_TITLE_STR      = resources.getString("APP_TITLE_STR");
    public static final String    AUTHOR_STR         = resources.getString("AUTHOR_STR");
    public static final String    DEC_FONT_STR       = resources.getString("DEC_FONT_STR");
    public static final String    INC_FONT_STR       = resources.getString("INC_FONT_STR");
    public static final String    COLOR_SELECT_STR   = resources.getString("COLOR_SELECT_STR");
    public static final String    CLEAR_ROOM_STR     = resources.getString("CLEAR_ROOM_STR");
    public static final String    DISCONNECT_STR     = resources.getString("DISCONNECT_STR");
    public static final String    CONNECT_STR        = resources.getString("CONNECT_STR");
    public static final String    CHOOSE_COLOR       = resources.getString("CHOOSE_COLOR");

    // buttons
    public static final String    CONNECT_BUTTON     = resources.getString("CONNECT_BUTTON");
    public static final String    DISCONNECT_BUTTON  = resources.getString("DISCONNECT_BUTTON");
    public static final String    CLEAR_BUTTON       = resources.getString("CLEAR_BUTTON");
    public static final String    COLOR_BUTTON       = resources.getString("COLOR_BUTTON");
    public static final String    LARGER_BUTTON      = resources.getString("LARGER_BUTTON");
    public static final String    SMALLER_BUTTON     = resources.getString("SMALLER_BUTTON");
    public static final String    ABOUT_BUTTON       = resources.getString("ABOUT_BUTTON");
    public static final String    EXIT_BUTTON        = resources.getString("EXIT_BUTTON");



	// instance variables
	private ChatRoomClient client;
	private JTextField text;
	private JTextArea chatLog;
	private String newLine;
	private int fontSize = 12;
	
	
	/**
	 * Constructs a new ChatRoomClientGUI.
	 */
	public ChatRoomClientGUI(ChatRoomClient client) {
		this.client = client;
		
		setLayout(new BorderLayout());
		
		// the text field for writing messages
		text = new JTextField();
		text.setForeground(Color.blue);
		text.setBorder(BorderFactory.createTitledBorder(
						   BorderFactory.createEtchedBorder(Color.black,Color.red),INPUT_MESSAGE));
		add(text,BorderLayout.SOUTH);
		
		
		// the text area for logging the chat discussions
		chatLog = new JTextArea(30,40);
		chatLog.setEditable(false);
		chatLog.setForeground(Color.black);
		chatLog.setBorder(BorderFactory.createTitledBorder(
							  BorderFactory.createEtchedBorder(Color.black,Color.red),CHAT_ROOM));
		
		add(new JScrollPane(chatLog), BorderLayout.CENTER);
		
		// the tool bar of the ui
		JToolBar toolbar = createToolBar();
		add(toolbar,BorderLayout.NORTH);
		
		newLine = System.getProperty("line.separator");
		createInputTextHandler();
	}
	
	/**
	 * Creates the tool bar for this UI
	 */
	private JToolBar createToolBar () {
		JToolBar toolbar = new JToolBar();
		toolbar.setFloatable(false);
		JButton connect    = new JButton(CONNECT_BUTTON,    CONNECT_IMAGE);
		JButton disconnect = new JButton(DISCONNECT_BUTTON, DISCONNECT_IMAGE);
		JButton clear      = new JButton(CLEAR_BUTTON,      CLEAR_IMAGE);
		JButton color      = new JButton(CLEAR_BUTTON,      COLOR_IMAGE);
		JButton plus       = new JButton(LARGER_BUTTON,     PLUS_IMAGE);
		JButton minus      = new JButton(SMALLER_BUTTON,    MINUS_IMAGE);
		JButton about      = new JButton(ABOUT_BUTTON,      ABOUT_IMAGE);
		JButton exit       = new JButton(EXIT_BUTTON,       EXIT_IMAGE);
		addToToolBar(
			toolbar,
			connect,
			new ActionListener() {
				public void actionPerformed (ActionEvent e) {
					client.connect();
				}
			},
			CONNECT_STR);

        addToToolBar(
			toolbar,
			disconnect,
			new ActionListener() {
				public void actionPerformed (ActionEvent e) {
					client.disconnect();
				}
			},
			DISCONNECT_STR);
		
		addToToolBar(
			toolbar,
			clear,
			new ActionListener() {
				public void actionPerformed (ActionEvent e) {
					clearChatLog();
				}
			},
			CLEAR_ROOM_STR);
		
		addToToolBar(
			toolbar,
			color,
			new ActionListener() {
				public void actionPerformed (ActionEvent e) {
					Color c = JColorChooser.showDialog(
						ChatRoomClientGUI.this,CHOOSE_COLOR,Color.black);
					if (c != null) {
						setColor(c);
					}
				}
			},
			COLOR_SELECT_STR);
		
		addToToolBar(
			toolbar,
			plus,
			new ActionListener() {
				public void actionPerformed (ActionEvent e) {
					setBiggerFont();
				}
			},
			INC_FONT_STR);
		
		addToToolBar(
			toolbar,
			minus,
			new ActionListener() {
				public void actionPerformed (ActionEvent e) {
					setSmallerFont();
				}
			},
			DEC_FONT_STR);

        addToToolBar(
			toolbar,
			about,
			new ActionListener() {
				public void actionPerformed (ActionEvent e) {
					JOptionPane.showMessageDialog(ChatRoomClientGUI.this,
												  AUTHOR_STR ,
												  APP_TITLE_STR,JOptionPane.INFORMATION_MESSAGE,
												  CLIENT_ICON);
				}
			},
			DISPLAY_INFO_STR);

		addToToolBar(
			toolbar,
			exit,
			new ActionListener() {
				public void actionPerformed (ActionEvent e) {
					exit();
				}
			},
			EXIT_CLIENT_STR);
		toolbar.setLayout(new GridLayout());
		return toolbar;
	}
	
	// Append a new line of text to the chat log
	public void addMessage(String text) {
		chatLog.append(text+newLine);
	}
	
	
	// handle the typed text and detect the 'enter' character.
	private void createInputTextHandler() {
		text.addKeyListener(new KeyAdapter() {
					public void keyPressed(KeyEvent e) {
						if (e.getKeyCode()==10) {
							sendMessage(text.getText());
							text.setText("");
						}
					}
				});
	}
	
	
	// sends a message to the server
	private void sendMessage(String message) {
		client.sendMessage
			(client.protocol.createChatMessage("<"+client.getUserName()+">"+message));
	}
	
	
	/**
	 * adds a given button to the given tool bar
	 * sets the buttons action listener and tool tip text
	 */
	private void addToToolBar (JToolBar toolbar,JButton button,  ActionListener listener, String tip) {
		toolbar.add(button);
		button.addActionListener(listener);
		button.setToolTipText(tip);
	}
	
	/**
	 * Returns the user name as entered by the user
	 */
	public String requestUserName () {
		return JOptionPane.showInputDialog(this,ENTER_NAME_STR,
										   SERVER_REQUEST_STR,
										   JOptionPane.INFORMATION_MESSAGE);
	}
	
	////////////////////////////////////////////////////////////////
	// Actions
	////////////////////////////////////////////////////////////////

	// called to exit the application
	protected void exit () {
		client.disconnect();
		System.exit(0);
	}
	
	
	// Clears the chat log
	private void clearChatLog() {
		chatLog.setText("");
	}
	
	
	// sets the color of the log display
	private void setColor(Color color) {
		chatLog.setForeground(color);
		chatLog.setText(chatLog.getText());
	}
	
	// Enlarge the text font size by 1
	private void setBiggerFont() {
		fontSize++;
		chatLog.setFont(new Font("Default",Font.PLAIN,fontSize));
	}
	
	// Reduce the text font size by 1
	private void setSmallerFont() {
		fontSize--;
		chatLog.setFont(new Font("Default",Font.PLAIN,fontSize));
	}
	
	/**
	 * Called to play a notification sound when the a user sends a message
	 */
	public void playChatSound () {
		CHAT_SOUND.play();
	}
    ////////////////////////////////////////////////////////////////
	// END Actions
	////////////////////////////////////////////////////////////////


	/**
	 * The Starting point of the application
	 */
	public static void main (String[] args) {
		int port;
		String address;
		ChatRoomClient client = null;
		JFrame frame = new JFrame ();
		frame.setIconImage(CLIENT_ICON.getImage());
		ConnectionInfoDialog dialog = new ConnectionInfoDialog(frame,CON_INFO_STR,true);
		dialog.pack();
		dialog.show();
		port = dialog.getPort();
		address = dialog.getAddress();
		client = new ChatRoomClient(address,port);
		final ChatRoomClientGUI ui = new ChatRoomClientGUI (client);
		client.setUI(ui);
		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				ui.exit();
			}
		});
		frame.getContentPane().add(ui);
		frame.pack();
		frame.setTitle(CHAT_ROOM_CLIENT);
		frame.show();
	}
}
