import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import java.net.*;
import java.applet.*;

/**
 * ChatRoomServerUI is a JPanel used for The UI for the ChatRoomServer.
 */
public class ChatRoomServerUI extends JPanel {

    public static ResourceManager resources = ResourceManager.getInstance();

    // the image files
    public static final ImageIcon CLEAR_IMAGE        = resources.getImageIcon("CLEAR_IMAGE");
    public static final ImageIcon CONTROL_IMAGE      = resources.getImageIcon("CONTROL_IMAGE");
    public static final ImageIcon CONNECTION_IMAGE   = resources.getImageIcon("CONNECTION_IMAGE");
    public static final ImageIcon LOG_IMAGE          = resources.getImageIcon("LOG_IMAGE");
    public static final ImageIcon CHAT_IMAGE         = resources.getImageIcon("CHAT_IMAGE");
    public static final ImageIcon ONLINE_IMAGE       = resources.getImageIcon("ONLINE_IMAGE");
    public static final ImageIcon OFFLINE_IMAGE      = resources.getImageIcon("OFFLINE_IMAGE");
    public static final ImageIcon START_SERVER_IMAGE = resources.getImageIcon("START_SERVER_IMAGE");
    public static final ImageIcon STOP_SERVER_IMAGE  = resources.getImageIcon("STOP_SERVER_IMAGE");
    public static final ImageIcon SERVER_ICON        = resources.getImageIcon("SERVER_ICON");

    //sounds
    public static final AudioClip DISCONNECTION_SOUND = resources.getAudioClip("DISCONNECTION_SOUND");
    public static final AudioClip CONNECTION_SOUND    = resources.getAudioClip("CONNECTION_SOUND");
    public static final AudioClip CHAT_SOUND          = resources.getAudioClip("CHAT_SOUND");


    // strings
    public static final String CONTROL_PANEL_TT       = resources.getString("CONTROL_PANEL_TT");
    public static final String CONTROL_PANEL          = resources.getString("CONTROL_PANEL");
    public static final String CONN_LOG               = resources.getString("CONN_LOG");
    public static final String EVENT_LOG_TT           = resources.getString("EVENT_LOG_TT");
    public static final String CLIENT_INFO_TT         = resources.getString("CLIENT_INFO_TT");
    public static final String CLIENT_INFO            = resources.getString("CLIENT_INFO");
    public static final String LOG_TT                 = resources.getString("LOG_TT");
    public static final String CHAT_ROOM              = resources.getString("CHAT_ROOM");
    public static final String START_BUTTON           = resources.getString("START_BUTTON");
    public static final String STOP_BUTTON            = resources.getString("STOP_BUTTON");
    public static final String CLEAR_LOG_BUTTON       = resources.getString("CLEAR_LOG_BUTTON");
    public static final String ONLINE_STR             = resources.getString("ONLINE_STR");
    public static final String OFFLINE_STR            = resources.getString("OFFLINE_STR");
    public static final String APP_TITLE              = resources.getString("APP_TITLE");

    private ChatRoomServer server;
    private ConnectionInformationTableModel model;

    // the UI panels
    private JTabbedPane tabbedPane;
    private JTable connectionTable;
    private JTextArea log;
    private JTextArea chatLog;

    // server status label
    JLabel serverStatus;

    /**
     * Constructs a new ChatRoom Server UI for a given chat room server.
     * @param	server	a ChatRoomServer.
     */
    public ChatRoomServerUI(ChatRoomServer server) {
        this.server = server;
        ImageIcon connectionTableIcon = CONTROL_IMAGE;
        ImageIcon logTextIcon         = LOG_IMAGE;
        ImageIcon informationIcon     = CONNECTION_IMAGE;
        ImageIcon chatIcon            = CHAT_IMAGE;
        serverStatus = new JLabel();
        setServerStatusText();


        // construct the connection table
        model = new ConnectionInformationTableModel(server);
        connectionTable =new JTable (model);
        JScrollPane scrollPane = new JScrollPane(connectionTable);

        // construct the logging text area
        log = new JTextArea(30,40);
        chatLog = new JTextArea(30,40);
        chatLog.setForeground(Color.blue);
        log.setEditable(false);
        chatLog.setEditable(false);

        JPanel logPanel = makeLogPanel(CONN_LOG, log);
        JPanel chatPanel = makeLogPanel(CHAT_ROOM, chatLog);

        // add panels to the tabbed pane
        tabbedPane = new JTabbedPane();
        tabbedPane.addTab(CONTROL_PANEL, connectionTableIcon, createControlPanel(),CONTROL_PANEL_TT);
        tabbedPane.addTab(CONN_LOG, logTextIcon, logPanel,EVENT_LOG_TT);
        tabbedPane.addTab(CLIENT_INFO, informationIcon, scrollPane,CLIENT_INFO_TT);
        tabbedPane.addTab(CHAT_ROOM, chatIcon, chatPanel,LOG_TT);
        tabbedPane.setSelectedIndex(0);

        //Add the tabbed pane to this panel.
        setLayout(new GridLayout(1,1));
        add(tabbedPane);
    }



    // Constructs the control panel for the server.
    private JPanel createControlPanel () {
        JPanel panel = new JPanel(false);
        panel.setLayout(new BorderLayout());

        JPanel buttons = new JPanel();
        JButton start = new JButton (START_BUTTON,START_SERVER_IMAGE);
        start.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (!server.isConnected()) {
                    server.start();
                }
                tabbedPane.setSelectedIndex(1);
            }
        });

        JButton stop = new JButton (STOP_BUTTON,STOP_SERVER_IMAGE);
        stop.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (server.isConnected()) {
                    server.stop();
                }
                tabbedPane.setSelectedIndex(1);
            }
        });
        buttons.add(start);
        buttons.add(stop);
        panel.add(buttons,BorderLayout.NORTH);

        JPanel status = new JPanel();
        status.setLayout(new FlowLayout(FlowLayout.CENTER,0,100));
        status.add(serverStatus);
        panel.add(status,BorderLayout.CENTER);
        return panel;
    }


    // creates a JPanel with a clear button to wrap the log text area

    private JPanel makeLogPanel(String text,final JTextArea logType) {
        JPanel panel = new JPanel(false);
        panel.setLayout(new BorderLayout());
        panel.add(new JScrollPane(logType), BorderLayout.CENTER);
        JButton clear = new JButton(CLEAR_LOG_BUTTON ,CLEAR_IMAGE);
        panel.add(clear, BorderLayout.SOUTH);
        clear.addActionListener(new ActionListener() {
            public void actionPerformed (ActionEvent e) {
                logType.setText(serverStatus.getText());
            }
        });
        return panel;
    }

    /**
     * Called to log a message to the logging text area.
     * @param	text	the message to log.
     */
    public synchronized void logMessage(String text) {
        log.append(text + "\n");
    }

    /**
     * Called to log a message to the logging text area.
     * @param	text	the message to log.
     */
    public synchronized void logError(String text) {
        log.append(text + "\n");
    }


    /**
     * Called to log a message to the chat logging text area.
     * @param	text	the message to log.
     */
    public void logChat(String text) {
        chatLog.append(text + "\n");
        playChatSound();
    }

    /**
     * Called to refresh the ui of the connection table.
     */
    public void refreshUI () {
        model.refresh();
        setServerStatusText();
    }

    // sets the text of the server status label
    private void setServerStatusText(){
        boolean connected = server.isConnected();
        if (connected) {
            serverStatus.setForeground(Color.green);
            serverStatus.setText(ONLINE_STR);
            serverStatus.setIcon(ONLINE_IMAGE);
        } else {
            serverStatus.setForeground(Color.red);
            serverStatus.setText(OFFLINE_STR);
            serverStatus.setIcon(OFFLINE_IMAGE);
        }

    }

    /**
     * Called to play a notification sound when the a user enters the chat room
     */
    public void playConnectionSound() {
        CONNECTION_SOUND.play();
    }


    /**
     * Called to play a notification sound when the a user leaves the chat room
     */
    public void playDisconnectionSound() {
        DISCONNECTION_SOUND.play();
    }


    /**
     * Called to play a notification sound when the a user sends a message
     */
    public void playChatSound () {
        CHAT_SOUND.play();
    }


    /**
     * Runs the server application
     */
    public static void main(String[] args) {
        ChatRoomServer server = new ChatRoomServer(9090);
        JFrame frame = new JFrame(APP_TITLE);
        frame.setIconImage(SERVER_ICON.getImage());
        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);}
        });
        ChatRoomServerUI ui = new ChatRoomServerUI(server);
        frame.getContentPane().add(ui,BorderLayout.CENTER);
        frame.setSize(700, 700);
        frame.setVisible(true);
        server.setLogger(ui);
    }
}
