import javax.swing.*;
import java.awt.*;
import java.net.URL;
import java.util.*;
import java.applet.Applet;
import java.applet.AudioClip;

public class ResourceManager
{

    public static final String DEFAULT_RESOURCES_LOCATION = "resources/";
    private static String resouceBundleName = "Chat";

    private static final ResourceManager instance = new ResourceManager();


    public ResourceManager () {
    }

    public static ResourceManager getInstance () {
        return instance;
    }


    public String getString( String label) {
        ResourceBundle bundle;
        try {
            bundle = ResourceBundle.getBundle(DEFAULT_RESOURCES_LOCATION + resouceBundleName);
        } catch (MissingResourceException mre) {
            mre.printStackTrace();
            return label;
        }
        try {
            String value = bundle.getString(label);

            if (value == null) {
                System.err.println("value (string) not found for the key " + label+" (at bundle '"+resouceBundleName+"')");
                return label;
            } else {
                //value = value.replaceAll("\\\\n", "\n");
                return value;
            }
        } catch (MissingResourceException mre) {
            System.err.println("Value (string) not found for the key " + label+" (at bundle '"+resouceBundleName+"')" + mre.getMessage());
            return label;
        }

    }

    public String[] getStrings(String label) {
        String result = getString(label);
        StringTokenizer tokenizer = new StringTokenizer(result,",");
        Vector v = new Vector();
        while (tokenizer.hasMoreElements()) {
            v.add(tokenizer.nextElement());
        }
        return (String[])v.toArray(new String[0]);
    }

    public Image getImage(String key) {
        try{
            URL url = fetchURL(key);
            return Toolkit.getDefaultToolkit().getImage(url);
        } catch (Throwable throwable) {
            throwable.printStackTrace();
            return null;
        }
    }

    public AudioClip getAudioClip(String key) {
        try{
            URL url = fetchURL(key);
            return Applet.newAudioClip(url);
        } catch (Throwable throwable) {
            throwable.printStackTrace();
            return null;
        }
    }

    private URL fetchURL(String key) throws Throwable {
        ResourceBundle bundle;
        try {
            bundle = ResourceBundle.getBundle(DEFAULT_RESOURCES_LOCATION + resouceBundleName);
        } catch (MissingResourceException mre) {
            throw new Throwable("Boundle "+DEFAULT_RESOURCES_LOCATION + resouceBundleName + " not found!");
        }

        String value;
        try {
            value = bundle.getString(key);
        } catch (MissingResourceException mre) {
            throw new Throwable("Value not found for the key " + key+" (at bundle '"+resouceBundleName+"')");
        }
        if (value == null) {
            throw new Throwable("Value not found for the key " + key+" (at bundle '"+resouceBundleName+"')");
        } else {
            String urlAsString = "/" + DEFAULT_RESOURCES_LOCATION + value;
            urlAsString = urlAsString.trim();
            URL url = ImageIcon.class.getResource(urlAsString);
            if (url == null) {
                throw new Throwable("Cannot create a URL object from the string: " + urlAsString+", (Is it missing ??)");
            } else {
                return url;
            }
        }
    }


    public ImageIcon getImageIcon(String key) {
        try {
            URL url = fetchURL(key);
            return new ImageIcon(url);
        } catch (Throwable throwable) {
            throwable.printStackTrace();
            return null;
        }
    }
}
